/*
 * @(#)MohrSolver.java
 */

/**
 * MohrSolver.java - Solver of Mohr Circle.
 *
 * <p>
 * Description:
 * <p>
 * ===============================================================
 * <p>
 * This file contains an implementation of the Mohr Circle for
 * plane stresses.  For more details of this process, refer to
 * book "Mecnica dos Slidos, Timoshenko & Gere".
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
public class MohrSolver
{
 /*
 ** ---------------------------------------------------------------
 ** Protected instance variables:
 */

 /** Tenso normal na direo x */
 protected double sigmax;

 /** Tenso normal na direo y */
 protected double sigmay;

 /** Tenso de cisalhamento */
 protected double tauxy;

 /** ngulo da normal do plano em relao ao eixo x (no sentido anti-horrio) */
 protected double theta;

 /*
 ** ---------------------------------------------------------------
 ** Package instance variables:
 */

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

 /*=======================  Constructor  =========================*/
 /**
  * Cria um objeto MohrSolver para um conjunto completo de dados.
  */
 public MohrSolver( double sigmax, double sigmay, 
                    double tauxy, double theta )
 {
  this.sigmax = sigmax;
  this.sigmay = sigmay;
  this.tauxy = tauxy;
  this.theta = theta;
 }

 /*=======================  Constructor  =========================*/
 /**
  * Cria um objeto MohrSolver com valores default.
  */
 public MohrSolver( )
 {
  this( 100.0, 50.0, 25.0, Math.toRadians( 52.0 ) );
 }

 /*======================  ChkNullStateStr  ========================*/
 /**
  * Verifica se as componentes de tenso passadas como parmetros
  * foram um estado de tenses nulo.
  */
 static protected boolean ChkNullStateStr( double sigmax, double sigmay, 
                                           double tauxy )
 {
  if( Math.sqrt(sigmax*sigmax + sigmay*sigmay + tauxy*tauxy) < 0.1 )
   return( true );

  return( false );
 }
 /*=========================  ResetSolver  =========================*/
 /**
  * Inicializa os parmetros do objeto MohrSolver com valores default.
  */
 public void ResetSolver()
 {
  sigmax = 100.0;
  sigmay = 50.0;
  tauxy = 25.0;
  theta = Math.toRadians( 52.0 );
 }

 /*========================  setSigmaX  =========================*/
 /**
  * Altera o valor corrente da tenso normal sigmax.
  */
 public void setSigmaX( double sigmax )
 {
  if( ChkNullStateStr( sigmax, this.sigmay, this.tauxy ) )
   return;

  this.sigmax = sigmax;
 }

 /*========================  setSigmaY  =========================*/
 /**
  * Altera o valor corrente da tenso normal sigmay.
  */
 public void setSigmaY( double sigmay )
 {
  if( ChkNullStateStr( this.sigmax, sigmay, this.tauxy ) )
   return;

  this.sigmay = sigmay;
 }

 /*========================  setTauXY  =========================*/
 /**
  * Altera o valor corrente da tenso de cisalhamento tauxy.
  */
 public void setTauXY( double tauxy )
 {
  if( ChkNullStateStr( this.sigmax, this.sigmay, tauxy ) )
   return;

  this.tauxy = tauxy;
 }

 /*========================  setTheta  =========================*/
 /**
  * Altera o valor corrente do ngulo da normal do plano de 
  * resposta (em radianos).  O valor do ngulo  sempre 
  * transformado para a faixa entre 0 e 180 graus.
  */
 public void setTheta( double theta )
 {
  int    ip;             /* integer part */
  double fp;             /* "fraction" part */

/* Primeiro corrige o ngulo para um valor com
 * mdulo menor que 360 graus.
 */
  fp = theta / (2.0*Math.PI);
  ip = (int)fp;
  theta = theta - (ip*2.0*Math.PI) ;

/* Agora transforma ngulo para a faixa entre 0 e 180 graus.
 */
  if( theta < 0.0 )
   theta = Math.PI + theta;
  else if( theta > Math.PI )
   theta = theta - Math.PI;

  this.theta = theta;
 }

 /*========================  getSigmaX  =========================*/
 /**
  * Retorna o valor corrente da tenso normal sigmax.
  */
 public double getSigmaX( )
 {
  return( sigmax );
 }

 /*========================  getSigmaY  =========================*/
 /**
  * Retorna o valor corrente da tenso normal sigmay.
  */
 public double getSigmaY( )
 {
  return( sigmay );
 }

 /*========================  getTauXY  =========================*/
 /**
  * Retorna o valor corrente da tenso de cisalhamento.
  */
 public double getTauXY( )
 {
  return( tauxy );
 }

 /*========================  getTheta  =========================*/
 /**
  * Retorna o valor corrente do ngulo que define a direo do
  * plano de resposta (em radianos).
  */
 public double getTheta( )
 {
  return( theta );
 }

 /*========================  Center  =========================*/
 /**
  * Retorna o valor mdio da tenso normal do estado corrente.
  */
 public double Center( )
 {
  return( (sigmax + sigmay) * 0.5 );
 }

 /*========================  Radius  =========================*/
 /**
  * Retorna o valor do raio do crculo de Mohr correspondente
  * ao estado de tenso corrente.
  */
 public double Radius( )
 {
  double sigmaux = Math.abs(sigmax - sigmay) * 0.5;

  return( Math.sqrt(sigmaux*sigmaux + tauxy*tauxy) );
 }

 /*========================  Sigma1  =========================*/
 /**
  * Retorna o valor da tenso normal mxima do estado corrente.
  */
 public double Sigma1( )
 {
  return( this.Center( ) + this.Radius( ) );
 }

 /*========================  Sigma2  =========================*/
 /**
  * Retorna o valor da tenso normal mnima do estado corrente.
  */
 public double Sigma2( )
 {
  return( this.Center( ) - this.Radius( ) );
 }

 /*========================  TauMax  =========================*/
 /**
  * Retorna o valor mximo em mdulo da tenso de cisalhamento.
  */
 public double TauMax( )
 {
  return( this.Radius( ) );
 }

 /*========================  SigmaTheta  =========================*/
 /**
  * Retorna o valor da tenso normal no plano cuja normal  defina
  * pelo valor corrente de theta.
  */
 public double SigmaTheta(  )
 {
  double sigmaux = (sigmax - sigmay) * 0.5;
  double thetaux = 2.0*theta;

  return( this.Center( )+sigmaux*Math.cos(thetaux)+tauxy*Math.sin(thetaux) );
 }

 /*========================  SigmaTheta90  =========================*/
 /**
  * Retorna o valor da tenso normal no plano cuja normal  defina
  * pelo valor corrente de theta + 90 graus.
  */
 public double SigmaTheta90(  )
 {
  double sigmaux = (sigmax - sigmay) * 0.5;
  double thetaux = 2.0*(theta+(Math.PI/4.0));

  return( this.Center( )+sigmaux*Math.cos(thetaux)+tauxy*Math.sin(thetaux) );
 }

 /*========================  TauTheta  =========================*/
 /**
  * Retorna o valor da tenso de cisalhamento no plano cuja normal
  *  defina pelo valor corrente de theta.
  */
 public double TauTheta(  )
 {
  double sigaux = (sigmax - sigmay) * 0.5;
  double thetaux = 2.0*theta;

  return( sigaux*Math.sin(thetaux) - tauxy*Math.cos(thetaux) );
 }

 /*========================  ThetaP  =========================*/
 /**
  * Retorna o valor do ngulo que a normal do plano onde 
  * ocorre a tenso normal principal mxima faz com o eixo x 
  * (em radianos).  O valor do ngulo  sempre transformado 
  * para a faixa entre 0 e 180 graus.
  */
 public double ThetaP(  )
 {
  double sigmaux = (sigmax - sigmay) * 0.5;
  double thetap;

  if( Math.abs( sigmaux ) > 0.0 )
   thetap = 0.5 * Math.atan2(tauxy,sigmaux);
  else if( tauxy > 0.0 )
   thetap = Math.PI / 4.0;
  else if( tauxy < 0.0 )
   thetap = -Math.PI / 4.0;
  else
   thetap = 0.0;

/* Transforma ngulo para a faixa entre 0 e 180 graus.
 */
  if( thetap < 0.0 )
   thetap = Math.PI + thetap;

  return( thetap );
 }

 /*=====================  IsHydrostatic  ======================*/
 /**
  * Retorna um flag para estado hidrosttico de tenses:
  * se verdadeiro, estado de tenses corrente  hidrosttico,
  * se falso, no  hidrosttico.
  */
 public boolean IsHydrostatic( )
 {
  int type;

  if( (Math.abs( sigmax - sigmay ) < 0.01) &&
      (Math.abs( tauxy ) < 0.01) )
  {
   return( true );
  }

  return( false );
 }

 /*========================  PoleX  ========================*/
 /**
  * Retorna coordenada horizontal do polo.
  */
 public double PoleX( )
 {
  return( sigmay );
 }

 /*========================  PoleY  ========================*/
 /**
  * Retorna coordenada vertical do polo.
  */
 public double PoleY( )
 {
  return( -tauxy );
 }

}

