/*
 * @(#)MohrStateStrsCanvas.java
 */

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;

import wc.*;

/**
 * MohrStateStrsCanvas.java - Desenha o estado de tenses.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a classe que desenha e manipula o 
 * estado de tenses.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrStateStrsCanvas extends CanvasWC
{
 /** Driver do programa Mohr-Circle */
 protected MohrDriver driver;

 /** Quadrado  */
 protected Rectangle2D.Double quad;
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico do quadrado */
 protected static final double QUAD_STROKE_FAC = 0.005;

 /** Objeto seta para desenhar tenses */
 protected Arrow2D arrow;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das setas */
 protected static final double ARROW_STROKE_FAC = 0.010;

 /** Stroke bsico dos eixos */
 protected BasicStroke arrow_stroke;

 /** Stroke bsico do quadrado */
 protected BasicStroke quad_stroke;

 /** Fonte adotado para desenho */
 protected Font fnt;


/*==========================  Constructor  =========================*/
/**
  * Constructor: Especifica eixo Y no sentido para cima.
  */
 MohrStateStrsCanvas( MohrDriver driver )
 {
  super( UPWARD_Y );
  this.driver = driver;

  double xmin = 0.0;
  double xmax = 600;
  double ymin = 0.0;
  double ymax = 600;
  
  double aresta = xmax/3.0;
  double centro = xmax/2.0;
  
  setWorld( xmin, xmax, ymin, ymax );

  quad = new Rectangle2D.Double( centro - (aresta/2.0), centro - (aresta/2.0), 
                                 aresta, aresta);
  quad_stroke = new BasicStroke( 
                               (float)(world_area.height*QUAD_STROKE_FAC) );
  arrow = new Arrow2D();
  arrow_stroke = new BasicStroke( 
                               (float)(world_area.height*ARROW_STROKE_FAC) );
                            
  fnt = new Font( "Times New Roman", Font.ITALIC, 14 );
 }

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */
 

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

/*===============================  update  ==============================*/
/**
  * Reset method.
  */
 void update()
 {
  double xmin = 0.0;
  double xmax = 600;
  double ymin = 0.0;
  double ymax = 600;
  
  double aresta = xmax/3.0;
  double centro = xmax/2.0;
  
  double fact = aresta/8.0;
  
  setWorld( xmin, xmax, ymin, ymax );
  
  quad.setFrame( centro - (aresta/2.0), centro - (aresta/2.0), aresta, aresta );

  quad_stroke = new BasicStroke( 
                               (float)(world_area.height*QUAD_STROKE_FAC) );

  arrow_stroke = new BasicStroke( 
                               (float)(world_area.height*ARROW_STROKE_FAC) );

  repaint();
 }

/*===============================  redraw  =============================*/
 /**
  * Redisplay method.
  * This method is called every time the canvas is updated.
  * Client's concrete class must implement it.
  * @param g is the 2D graphics context that the client uses to display
  */
 public void redraw( Graphics2D g )
 {
  double pi = Math.PI;
  double xmin = world_area.x;
  double xmax = world_area.x + world_area.width;
  double ymin = world_area.y;
  double ymax = world_area.y + world_area.height;

  double aresta = (ymax-ymin)/3.0;
  double centro = (xmax+xmin)/2.0;
  
  double fact = aresta/8.0;
  
  double x = centro;
  double y = centro;

  g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                      RenderingHints.VALUE_ANTIALIAS_ON );
  
  g.setFont( fnt );                    

  g.setColor( Color.white );
  g.fill( quad );
  g.setColor( Color.black );
  g.draw( quad );
  
  g.setStroke( arrow_stroke );
  
  /* Desenha Sigma X */
  if( Math.abs( driver.getSolver().getSigmaX() ) > 0.01 )
  {
   g.setColor( Color.blue.darker() );
   if (driver.getSolver().getSigmaX() > 0)
   {
    x = centro + (aresta/2.0) + fact;
    y = centro;
    arrow.DrawArrow( g, x, y, 0.0, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
	   
    x = centro - (aresta/2.0) - fact;
    y = centro;
    arrow.DrawArrow( g, x, y, pi, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
   }
   else if(driver.getSolver().getSigmaX() < 0)
   {
    x = centro + (aresta/2.0) + fact;
    y = centro;
    arrow.DrawArrow( g, x, y, 0.0, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   
    x = centro - (aresta/2.0) - fact;
    y = centro;
    arrow.DrawArrow( g, x, y, pi, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   }
  }

  /* Desenha Sigma Y */
  if( Math.abs( driver.getSolver().getSigmaY() ) > 0.01 )
  {
   g.setColor( Color.blue.brighter() );
   if (driver.getSolver().getSigmaY() > 0)
   {
    x = centro;
    y = centro + (aresta/2.0) + fact;
    arrow.DrawArrow( g, x, y, pi/2, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
   
    x = centro;
    y = centro - (aresta/2.0) - fact;
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
   }
   else if(driver.getSolver().getSigmaY() < 0)
   {
    x = centro;
    y = centro + (aresta/2.0) + fact;
    arrow.DrawArrow( g, x, y, pi/2, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   
    x = centro;
    y = centro - (aresta/2.0) - fact;
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   }
  }

  /* Desenha Tau XY */
  if( Math.abs( driver.getSolver().getTauXY() ) > 0.01 )
  {
   if (driver.getSolver().getTauXY() > 0)
   {
    g.setColor( Color.blue.brighter() );	 
    x = centro - (aresta/2.0);
    y = centro + (aresta/2.0) + (fact/1.5);
    arrow.DrawArrow( g, x, y, 0.0, aresta, 
		     Arrow2D.SIDE_LEAD_LEFT, pi/4.0, aresta/10.0 );
	   
    g.setColor( Color.blue.darker());
    x = centro + (aresta/2.0) + (fact/1.5);
    y = centro - (aresta/2.0);
    arrow.DrawArrow( g, x, y, pi/2, aresta, 
		     Arrow2D.SIDE_LEAD_RIGHT, pi/4.0, aresta/10.0 );
	   
    g.setColor( Color.blue.brighter() );
    x = centro + (aresta/2.0);
    y = centro - (aresta/2.0) - (fact/1.5);
    arrow.DrawArrow( g, x, y, pi, aresta, 
    Arrow2D.SIDE_LEAD_LEFT, pi/4.0, aresta/10.0 );
       
    g.setColor( Color.blue.darker() );
    x = centro - (aresta/2.0) - (fact/1.5);
    y = centro + (aresta/2.0);
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta, 
    Arrow2D.SIDE_LEAD_RIGHT, pi/4.0, aresta/10.0 );
   }
   else if(driver.getSolver().getTauXY() < 0)
   {
    g.setColor( Color.blue.brighter() );
    x = centro - (aresta/2.0);
    y = centro + (aresta/2.0) + (fact/1.5);
    arrow.DrawArrow( g, x, y, 0.0, aresta, 
		     Arrow2D.SIDE_TRAIL_LEFT, pi/4.0, aresta/10.0 );
	   
    g.setColor( Color.blue.darker() );
    x = centro + (aresta/2.0) + (fact/1.5);
    y = centro - (aresta/2.0);
    arrow.DrawArrow( g, x, y, pi/2, aresta, 
		     Arrow2D.SIDE_TRAIL_RIGHT, pi/4.0, aresta/10.0 );
	   
    g.setColor( Color.blue.brighter() );
    x = centro + (aresta/2.0);
    y = centro - (aresta/2.0) - (fact/1.5);
    arrow.DrawArrow( g, x, y, pi, aresta, 
		     Arrow2D.SIDE_TRAIL_LEFT, pi/4.0, aresta/10.0 );
       
    g.setColor( Color.blue.darker() );
    x = centro - (aresta/2.0) - (fact/1.5);
    y = centro + (aresta/2.0);
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta, 
		     Arrow2D.SIDE_TRAIL_RIGHT, pi/4.0, aresta/10.0 );
   }
  }
  
  g.setColor( Color.black );
  setTextAlignment( CENTER );

  if( Math.abs( driver.getSolver().getSigmaX() ) > 0.01 )
  {
   drawString( "\u03C3x", centro + aresta + (2.5*fact), centro );
   drawString( "\u03C3x", centro - (aresta + (2.5*fact)), centro );
  }
  
  if( Math.abs( driver.getSolver().getSigmaY() ) > 0.01 )
  {
   drawString( "\u03C3y", centro, centro + aresta + (2.5*fact) );
   drawString( "\u03C3y", centro, centro - (aresta + (2.5*fact)));
  }

  if( Math.abs( driver.getSolver().getTauXY() ) > 0.01 )
  {
   if (driver.getSolver().getTauXY() > 0)
   {
    drawString( "\u03C4xy", centro + ((aresta/2.0) + (2.5*fact)), 
			    centro +  (2.0*fact) );
    drawString( "\u03C4xy", centro +  (2.0*fact), 
			    centro + ((aresta/2.0) + (2.0*fact)) );
    drawString( "\u03C4xy", centro - ((aresta/2.0) + (2.5*fact)), 
			    centro -  (2.0*fact) );
    drawString( "\u03C4xy", centro -  (2.0*fact), 
			    centro - ((aresta/2.0) + (2.0*fact)) );
   }
   else if(driver.getSolver().getTauXY() < 0)
   {
    drawString( "\u03C4xy", centro + ((aresta/2.0) + (2.5*fact)), 
			    centro -  (2.0*fact) );
    drawString( "\u03C4xy", centro +  (2.0*fact), 
			    centro - ((aresta/2.0) + (2.0*fact)) );
    drawString( "\u03C4xy", centro - ((aresta/2.0) + (2.5*fact)), 
			    centro +  (2.0*fact) );
    drawString( "\u03C4xy", centro -  (2.0*fact), 
			    centro + ((aresta/2.0) + (2.0*fact)) );
   }
  }
 }

 /**
  * Handles the event of the user pressing down the mouse button.
  */
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }

 /**
  * Handles the event of a user dragging the mouse while holding 
  * down the mouse button.
  */
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }

 /**
  * Handles the event of a user releasing the mouse button.
  */
  public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }
}

