/*
 * @(#)MohrThetaStrsCanvas.java
 */

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;

import wc.*;

/**
 * MohrThetaStrsCanvas.java - Desenha o estado de tenses.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a classe que desenha e manipula o 
 * estado de tenses para o plano theta.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrThetaStrsCanvas extends CanvasWC
{
 /** Driver do programa Mohr-Circle */
 protected MohrDriver driver;

 /** Quadrado  */
 protected Rectangle2D.Double quad;
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico do quadrado */
 protected static final double QUAD_STROKE_FAC = 0.005;

 /** Objeto seta para desenhar tenses */
 protected Arrow2D arrow;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das setas */
 protected static final double ARROW_STROKE_FAC = 0.010;

 /** Stroke bsico dos eixos */
 protected BasicStroke arrow_stroke;

 /** Stroke bsico do quadrado */
 protected BasicStroke quad_stroke;

 /** Objeto arco para desenhar ngulos */
 protected MohrArc arc;

 /** Objeto linha para desenho de linhas auxiliares */

 protected Line2D.Double auxline;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das linhas auxiliares */
 protected static final double AUXLINE_STROKE_FAC = 0.010;
 
 /** Stroke bsico das linhas auxiliares */
 protected BasicStroke auxline_stroke;

 /** Transformao window-viewport corrent */
 protected AffineTransform def_transf;

 /** Fonte adotado para desenho */
 protected Font fnt;


/*==========================  Constructor  =========================*/
/**
  * Constructor: Especifica eixo Y no sentido para cima.
  */
 MohrThetaStrsCanvas( MohrDriver driver )
 {
  super( UPWARD_Y );
  this.driver = driver;

  double xmin = 0.0;
  double xmax = 600.0;
  double ymin = 0.0;
  double ymax = 600.0;
  
  double aresta = xmax/3.0;
  double centro = xmax/2.0;

  setWorld( xmin, xmax, ymin, ymax );

  quad = new Rectangle2D.Double( centro - (aresta/2), centro - (aresta/2), 
                                 aresta, aresta);
  quad_stroke = new BasicStroke( 
                               (float)(world_area.height*QUAD_STROKE_FAC) );
  arrow = new Arrow2D();
  arrow_stroke = new BasicStroke( 
                               (float)(world_area.height*ARROW_STROKE_FAC) );

  arc = new MohrArc( );

  auxline = new Line2D.Double( );
  auxline_stroke = new BasicStroke( 
                            (float)(world_area.height*AUXLINE_STROKE_FAC) );

  def_transf = new AffineTransform( );

  fnt = new Font( "Times New Roman", Font.ITALIC, 14 );
 }

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */
 

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

/*===============================  update  ==============================*/
/**
  * Reset method.
  */
 void update()
 {
  
  double xmin = 0.0;
  double xmax = 600.0;
  double ymin = 0.0;
  double ymax = 600.0;
  
  double aresta = xmax/3.0;
  double centro = xmax/2.0;
    
  setWorld( xmin, xmax, ymin, ymax );

  quad.setFrame( centro - (aresta/2), centro - (aresta/2), aresta, aresta );
  
  quad_stroke = new BasicStroke( 
                               (float)(world_area.height*QUAD_STROKE_FAC) );

  arrow_stroke = new BasicStroke( 
                               (float)(world_area.height*ARROW_STROKE_FAC) );

  auxline_stroke = new BasicStroke( 
                            (float)(world_area.height*AUXLINE_STROKE_FAC) );

  repaint();
 }

/*===============================  redraw  =============================*/
 /**
  * Redisplay method.
  * This method is called every time the canvas is updated.
  * Client's concrete class must implement it.
  * @param g is the 2D graphics context that the client uses to display
  */
 public void redraw( Graphics2D g )
 {
  double pi = Math.PI;
  double xmin = world_area.x;
  double xmax = world_area.x + world_area.width;
  double ymin = world_area.y;
  double ymax = world_area.y + world_area.height;

  double aresta = (ymax-ymin)/3.0;
  double centro = (xmax+xmin)/2.0;
  
  double fact = aresta/8.0;
  
  double x = centro;
  double y = centro;

  double sin_a;
  double cos_a;

  g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                      RenderingHints.VALUE_ANTIALIAS_ON );

  g.setFont( fnt );

  /* Desenha arco com ngulo */
  if( driver.getSolver().getTheta() > 0.2 )
  {
   g.setColor( Color.gray.brighter() );
   g.setStroke( auxline_stroke );
   auxline.setLine( centro + 0.5*aresta, centro, centro + 1.1*aresta, centro );
   g.draw( auxline );
   cos_a = Math.cos( driver.getSolver().getTheta() );
   sin_a = Math.sin( driver.getSolver().getTheta() );
   arc.DrawArc( this, g, Color.gray.brighter(),
                centro, centro, centro + aresta, centro,
                centro + aresta*cos_a, centro + aresta*sin_a,
                aresta, "\u03B8", 1.5*fact, auxline_stroke );
  }

  /* Ajusta transformao de rotao */
  def_transf = g.getTransform();
  AffineTransform curr_transf = g.getTransform();

  curr_transf.translate( centro, centro );
  curr_transf.rotate( driver.getSolver().getTheta() );
  curr_transf.translate( -centro, -centro );
  g.setTransform( curr_transf );

  g.setColor( Color.white );
  g.fill( quad );
  g.setColor( Color.black );
  g.draw( quad );
  
  g.setStroke( arrow_stroke );
  
  /* Desenha Sigma Theta */
  if( Math.abs( driver.getSolver().SigmaTheta() ) > 0.01 )
  {
   g.setColor( Color.green.darker() );
   if( driver.getSolver().SigmaTheta() > 0 )
   {
    x = centro + (aresta/2.0) + fact;
    y = centro;
    arrow.DrawArrow( g, x, y, 0.0, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
	   
    x = centro - (aresta/2.0) - fact;
    y = centro;
    arrow.DrawArrow( g, x, y, pi, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
   }
   else if( driver.getSolver().SigmaTheta() < 0 )
   {
    x = centro + (aresta/2.0) + fact;
    y = centro;
    arrow.DrawArrow( g, x, y, 0.0, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   
    x = centro - (aresta/2.0) - fact;
    y = centro;
    arrow.DrawArrow( g, x, y, pi, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   }
  }

  /* Desenha Sigma Theta+90 */
  if( Math.abs( driver.getSolver().SigmaTheta90() ) > 0.01 )
  {
   g.setColor( Color.gray );
   if( driver.getSolver().SigmaTheta90() > 0 )
   {
    x = centro;
    y = centro + (aresta/2.0) + fact;
    arrow.DrawArrow( g, x, y, pi/2, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
   
    x = centro;
    y = centro - (aresta/2.0) - fact;
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta/2.0, 
		     Arrow2D.SIDE_LEAD, pi/4.0, aresta/10.0 );
   }
   else if( driver.getSolver().SigmaTheta90() < 0 )
   {
    x = centro;
    y = centro + (aresta/2.0) + fact;
    arrow.DrawArrow( g, x, y, pi/2, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   
    x = centro;
    y = centro - (aresta/2.0) - fact;
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta/2.0, 
		     Arrow2D.SIDE_TRAIL, pi/4.0, aresta/10.0 );
   }
  }

  /* Desenha Tau Theta */
  if( Math.abs( driver.getSolver().TauTheta() ) > 0.01 )
  {
   if( driver.getSolver().TauTheta() < 0 )
   {
    g.setColor( Color.green.darker() );
    x = centro + (aresta/2.0) + (fact/1.5);
    y = centro - (aresta/2.0);
    arrow.DrawArrow( g, x, y, pi/2, aresta, 
		     Arrow2D.SIDE_LEAD_RIGHT, pi/4.0, aresta/10.0 );

    x = centro - (aresta/2.0) - (fact/1.5);
    y = centro + (aresta/2.0);
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta, 
		     Arrow2D.SIDE_LEAD_RIGHT, pi/4.0, aresta/10.0 );

    g.setColor( Color.gray );
    x = centro - (aresta/2.0);
    y = centro + (aresta/2.0) + (fact/1.5);
    arrow.DrawArrow( g, x, y, 0.0, aresta, 
		     Arrow2D.SIDE_LEAD_LEFT, pi/4.0, aresta/10.0 );

    x = centro + (aresta/2.0);
    y = centro - (aresta/2.0) - (fact/1.5);
    arrow.DrawArrow( g, x, y, pi, aresta, 
		     Arrow2D.SIDE_LEAD_LEFT, pi/4.0, aresta/10.0 );
   }
   else if( driver.getSolver().TauTheta() > 0 )
   {
    g.setColor( Color.green.darker() );
    x = centro + (aresta/2.0) + (fact/1.5);
    y = centro - (aresta/2.0);
    arrow.DrawArrow( g, x, y, pi/2, aresta, 
		     Arrow2D.SIDE_TRAIL_RIGHT, pi/4.0, aresta/10.0 );

    x = centro - (aresta/2.0) - (fact/1.5);
    y = centro + (aresta/2.0);
    arrow.DrawArrow( g, x, y, 3*pi/2, aresta, 
		     Arrow2D.SIDE_TRAIL_RIGHT, pi/4.0, aresta/10.0 );

    g.setColor( Color.gray );
    x = centro - (aresta/2.0);
    y = centro + (aresta/2.0) + (fact/1.5);
    arrow.DrawArrow( g, x, y, 0.0, aresta, 
		     Arrow2D.SIDE_TRAIL_LEFT, pi/4.0, aresta/10.0 );

    x = centro + (aresta/2.0);
    y = centro - (aresta/2.0) - (fact/1.5);
    arrow.DrawArrow( g, x, y, pi, aresta, 
		     Arrow2D.SIDE_TRAIL_LEFT, pi/4.0, aresta/10.0 );
   }
  }

  g.setColor( Color.black );
  setTextAlignment( CENTER );

  if( Math.abs( driver.getSolver().SigmaTheta() ) > 0.01 )
  {
   drawString( "\u03C3\u03B8", centro + aresta + (2.5*fact), centro );
   drawString( "\u03C3\u03B8", centro - (aresta + (2.5*fact)), centro );
  }

  if( Math.abs( driver.getSolver().TauTheta() ) > 0.01 )
  {
   if( driver.getSolver().TauTheta() < 0 )
   {
    drawString( "\u03C4\u03B8", centro + ((aresta/2.0) + (2.3*fact)), 
			   centro +  (2.0*fact) );
    drawString( "\u03C4\u03B8", centro - ((aresta/2.0) + (2.3*fact)), 
			   centro -  (2.0*fact) );
   }
   else if( driver.getSolver().TauTheta() > 0 )
   {
    drawString( "\u03C4\u03B8", centro + ((aresta/2.0) + (2.3*fact)), 
			   centro -  (2.0*fact) );
    drawString( "\u03C4\u03B8", centro - ((aresta/2.0) + (2.3*fact)), 
			   centro +  (2.0*fact) );
   }
  }

  g.setTransform( def_transf );
 }

 /**
  * Handles the event of the user pressing down the mouse button.
  */
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }

 /**
  * Handles the event of a user dragging the mouse while holding 
  * down the mouse button.
  */
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }

 /**
  * Handles the event of a user releasing the mouse button.
  */
  public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
 }
}

