/*
 * @(#)CanvasWC.java	 1.0 06/11/1999
 */

package wc;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.font.*;
import javax.swing.JPanel;

/**
 * This <code>CanvasWC</code> class extends the {@link JPanel} class to 
 * provide control over displaying using a device-independent coordinate 
 * system called World Coordinates (WC), which is used by applications.
 * Displaying is buffered to a {@link BufferedImage} object (offscreen 
 * image) to emulate double buffering rendering. The <code>BufferedImage</code> 
 * object contains a {@link Graphics2D} object as part of its rendering state.
 * The <code>Graphics2D</code> object contains an {@link AffineTransform} 
 * object that defines how to convert coordinates from client space (world 
 * coordinates) to device-dependent (raster) coordinates in device space.
 * <p>
 * Clients need to extend this class, by creating subclasses, in order to 
 * use it.
 * <p>
 * The vertical Y axis of the <code>CanvasWC</code> object is set by default 
 * in downward direction, but clients may specify it in upward direction 
 * when the canvas is created.
 * <p>
 * ===================
 * @see        javax.swing.JPanel
 * @see        java.awt.Graphics2D
 * @see        java.awt.image.BufferedImage
 * @see        java.awt.event.MouseListener
 * @see        java.awt.event.MouseMotionListener
 * @version    1.1 02/08/2000
 * @author     Luiz Fernando Martha
 */
// Change history:  Luiz Fernando Martha 17/02/2000:
//   Added drawText and setTextAlignment methods.
// Change history:  Luiz Fernando Martha 04/04/2000:
//   Added setTextOpacity method.
// Change history:  Luiz Fernando Martha 02/08/2000:
//   Replaced super-class of CanvasWC from Canvas to
//   JPanel.  This was done because class Canvas 
//   is an AWT class and JPanel is a Swing class.
//   When CanvasWC was an extension of Canvas, some  
//   conflicts with other Swing UI objects were occuring,
//   such as a dropdown object was drawn on the back 
//   of the canvas instead of on the front of it.
//   Commented out method paint.  Replaced method 
//   update by method paintComponet (with the same 
//   Graphics g argument).

public abstract class CanvasWC extends JPanel implements MouseListener, MouseMotionListener
{
 /**
  * Defines the canvas raster area. This area corresponds to the sizes of the 
  * canvas in pixels.
  * @serial
  * @see java.awt.Dimension
  */
 protected Dimension raster_area;

 /**
  * Defines the canvas world coordinate area. This area corresponds to the 
  * window in client space which is visible in the canvas.  This window is
  * adjusted to have the same aspect ratio between height and width of the
  * canvas (in its current state).  When the client specifies the windows,
  * the adjustment maintains the center of given (raw) window in the same 
  * position.  This adjustment may be affected by the canvas vertical 
  * distortion factor. 
  * @serial
  * @see java.awt.geom.Rectangle2D.Double
  * @see #vdistortion
  * @see #setWorld
  */
 protected Rectangle2D.Double world_area;

 /**
  * Defines the original canvas world coordinate area. This area corresponds 
  * to the window in world coordinates which was set by the client.  That is,
  * it corresponds to the world area prior to adjustment with respect to the 
  * canvas aspect ratio between height and width. 
  * @serial
  * @see #setWorld
  * @see #world_area
  */
 protected Rectangle2D.Double world_raw;

 /**
   * Field constant used to define the vertical Y axis of the canvas in 
   * downward direction.
   */
 public static final int DOWNWARD_Y = 0;

 /**
   * Field constant used to define the vertical Y axis of the canvas in 
   * upward direction.
   */
 public static final int UPWARD_Y = 1;

 /**
  * Keeps the vertical Y axis direction state.  The possible values are 
  * <code>DOWNWARD_Y</code> (default) or <code>UPWARD_Y</code>.  The 
  * Y axis direction is specified during creation, when the
  * <code>CanvasWC</code> constructor is called.
  * @serial
  */
 protected int y_dir;

 /**
  * BufferedImage object that is used to implement double
  * buffering rendering.  This object defines a Graphics2D 
  * graphics context object used to convert from world coordinates 
  * to raster coordinates.
  * @serial
  */
 private BufferedImage ioff;

 /**
  * Graphics context object (Graphics) of the offscreen
  * image used privately for double buffering.
  * @serial
  */
 private Graphics goff;

 /**
  * Graphics context object (Graphics2D) that contains an 
  * AffineTransform object that defines how to convert 
  * coordinates from client space (world coordinates) to device-dependent 
  * (raster) coordinates in device space.
  * @serial
  */
 private Graphics2D g2off;

 /**
  * Keeps the WC window vertical distortion factor.  This factor specifies
  * the amount of distortion between the height and width of the world
  * coordinate area. It is essentially a factor between the vertical and
  * horizontal scales in client world coordinate space. A factor with 
  * value greater than one will cause a vertical stretching of the 
  * image.
  * @serial
  * @see #setVDistortion
  * @see #world_area
  */
 protected double vdistortion;

 /**
  * Private field that holds a auxiliary point in world coordinates.
  * @serial
  */
 private Point2D.Double pt_world;

 /**
  * Private field that holds a auxiliary point in raster (device) coordinates.
  * @serial
  */
 private Point pt_raster;

 /**
  * Private field that holds a mouse event point in world coordinates.
  * @serial
  */
 private Point2D.Double ept_world;

 /**
  * Private field that holds a mouse event point in raster (device) coordinates.
  * @serial
  */
 private Point ept_raster;

 /**
   * Field constant used to define transparent text background opacity.
   */
 public static final int TRANSPARENT = 0;

 /**
   * Field constant used to define opaque text background opacity.
   */
 public static final int OPAQUE = 1;

 /**
  * Private field that holds current text alignment parameter.
  */
 private int text_opacity = TRANSPARENT;

 /**
  * Private field that holds text box for opaque displaying.
  */
 private Rectangle text_box;

 /**
   * Field constant used to define text alignment.
   */
 public static final int NORTH_WEST = 0;

 /**
   * Field constant used to define text alignment.
   */
 public static final int NORTH = 1;

 /**
   * Field constant used to define text alignment.
   */
 public static final int NORTH_EAST = 2;

 /**
   * Field constant used to define text alignment.
   */
 public static final int SOUTH_WEST = 3;

 /**
   * Field constant used to define text alignment.
   */
 public static final int SOUTH = 4;

 /**
   * Field constant used to define text alignment.
   */
 public static final int SOUTH_EAST = 5;

 /**
   * Field constant used to define text alignment.
   */
 public static final int WEST = 6;

 /**
   * Field constant used to define text alignment.
   */
 public static final int CENTER = 7;

 /**
   * Field constant used to define text alignment.
   */
 public static final int EAST = 8;

 /**
   * Field constant used to define text alignment.
   */
 public static final int BASE_LEFT = 9;

 /**
   * Field constant used to define text alignment.
   */
 public static final int BASE_CENTER = 10;

 /**
   * Field constant used to define text alignment.
   */
 public static final int BASE_RIGHT = 11;

 /**
  * Private field that holds current text alignment parameter.
  */
 private int text_alignment = BASE_LEFT;

 /**
  * Private field that holds current font that is used to display text.
  */
 private Font fnt;

 /**
  * Font renderer context: private field that is a container for 
  * the information needed to measure text that is displayed.
  */
 private FontRenderContext frc;

 /**
  * Private field that holds an auxiliary identity affine 
  * transformation matrix.
  */
  AffineTransform identity;

 /**
  * Default constructor.  Sets up vertical Y axis in downward direction.
  */
 public CanvasWC()
 {
  this( DOWNWARD_Y );
 }

 /**
  * Constructor that sets up WC canvas according to a given orientation.
  * @param y_dir can either be <code>DOWNWARD_Y</code> or <code>UPWARD_Y</code>
  * @see #DOWNWARD_Y
  * @see #UPWARD_Y
  */
 public CanvasWC( int y_dir )
 {
  this.y_dir = y_dir;
  vdistortion = 1.0;
  pt_world = new Point2D.Double();
  pt_raster = new Point();
  ept_world = new Point2D.Double();
  ept_raster = new Point();
  addMouseMotionListener( this );
  addMouseListener( this );

  text_box = new Rectangle();

  // Create default container of font size information
  identity = new AffineTransform();
  frc = new FontRenderContext( identity, false, false );
 }

 /**
  * Redisplay abstract method.  Client's concrete class must implement it.
  * This method is called every time the canvas is updated.
  * @param g is the 2D graphics context that the client uses to display
  */
 public abstract void redraw( Graphics2D g );

 /**
  * Mouse button pressed abstract method.  Client's concrete class must 
  * implement it.  This method is called when the user presses a mouse 
  * button in the canvas.
  * @param e is mouse event object
  * @param pt is the mouse position in client's floating point world coordinates
  */
 public abstract void mousePressedWC( MouseEvent e, Point2D.Double pt );

 /**
  * Mouse button dragged abstract method.  Client's concrete class must 
  * implement it.  This method is called when user drags the mouse in the 
  * canvas with a button pressed.
  * @param e is mouse event object
  * @param pt is the mouse position in client's floating point world coordinates
  */
 public abstract void mouseDraggedWC ( MouseEvent e, Point2D.Double pt );

 /**
  * Mouse button released abstract method.  Client's concrete class must 
  * implement it.  This method is called when user the mouse button at the
  * end of a mouse pressed-dragged-released interaction.
  * @param e is mouse event object
  * @param pt is the mouse position in client's floating point world coordinates
  */
 public abstract void mouseReleasedWC( MouseEvent e, Point2D.Double pt );

 /**
  * Creates the BufferedImage object that is used to implement double
  * buffering rendering.  This object defines a Graphics2D graphics 
  * context object used to convert from world coordinates 
  * to raster coordinates.
  */
 private void buildOffscreen()
 {
  Dimension dim = getSize();
  int w = dim.width;
  int h = dim.height;
  raster_area = new Dimension( w, h );
  Image img = createImage( w, h );
  goff = img.getGraphics();
  ioff = (BufferedImage)img;
  if( g2off != null ) g2off.dispose();
  g2off = ioff.createGraphics();
  if( fnt != null )
   g2off.setFont( fnt );
 }

 /**
  * Adjusts the window in world coordinates (world_area) to have the 
  * same aspect ratio between height and width of the canvas (in its 
  * current state).
  * The adjustment maintains the center of given (raw) window in the same 
  * position.  This adjustment may be affected by the canvas vertical 
  * distortion factor (vdistortion).
  */
 private void adjustWorld()
 {
  double  vpr;               // canvas viewport aspect ratio
  double  xc, yc;            // center of world area

  vpr = (double)raster_area.height / (double)raster_area.width;
  xc = world_area.x + (world_area.width * 0.5);
  yc = world_area.y + (world_area.height * 0.5);
  if( (world_area.height*vdistortion) > (vpr*world_area.width) )
   world_area.width = (world_area.height*vdistortion) / vpr;
  else
   world_area.height = (world_area.width/vdistortion) * vpr;
  world_area.x = xc - (world_area.width * 0.5);
  world_area.y = yc - (world_area.height * 0.5);

  double   sx, sy, tx, ty;    // WC to raster affine transf. parameters
  sx = (double)raster_area.width / world_area.width;
  sy = (double)raster_area.height / world_area.height;
  tx = - (sx * (double)world_area.x);
  ty = - (sy * (double)world_area.y);

  AffineTransform wc2rc = new AffineTransform();
  if( y_dir == UPWARD_Y )
   wc2rc.setTransform( sx, 0.0, 0.0, -sy, tx, raster_area.height - ty );
  else
   wc2rc.setTransform( sx, 0.0, 0.0, sy, tx, ty );
  g2off.setTransform( wc2rc );
 }

 /**
  * Sets the limits of the world coordinate window for the target canvas.
  * The limits are adjusted to be consistent with the canvas
  * viewport aspect ratio and with the canvas vertical distortion 
  * factor.  The adjustment is such that the center of the given
  * window is maintained.
  * <p>
  * The canvas is not automatically repainted by this method.
  * @param xmin is the left limit of target window
  * @param xmax is the right limit of target window
  * @param ymin is the bottom limit of target window if vertical Y axis is upward, otherwise it is the top limit
  * @param ymax is the top limit of target window if vertical Y axis is upward, otherwise it is the bottom limit
  * @see #world_area
  * @see #setVDistortion
  */
 public void setWorld( double xmin, double xmax, double ymin, double ymax )
 {
  if( world_area == null )
  {
   world_raw = new Rectangle2D.Double();
   world_area = new Rectangle2D.Double();
  }
  world_raw.x = world_area.x = xmin;
  world_raw.y = world_area.y = ymin;
  world_raw.width = world_area.width = xmax - xmin;
  world_raw.height = world_area.height = ymax - ymin;
  if( g2off != null )
   adjustWorld();
 }

 /**
  * Translates the world coordinate window of the target canvas in the 
  * horizontal direction based on a given pan factor.
  * This factor is a percentage of the current window horizontal 
  * size.  A negative factor indicates a window translation to 
  * the right (the client model will move left), and a positive 
  * factor indicates a window translation to the left (the client
  * model will move right).
  * <p>
  * The canvas is not automatically repainted by this method.
  * @param fac is the pan factor
  * @see #world_area
  * @see #setWorld
  */
 public void panWorldX( double fac )
 {
  double translation;            // window horizontal translation
  double xmin, xmax, ymin, ymax; // new window limits

  translation = world_area.width * fac;
  xmin = world_area.x - translation;
  xmax = world_area.x + world_area.width - translation;
  ymin = world_area.y;
  ymax = world_area.y + world_area.height;
  setWorld( xmin, xmax, ymin, ymax );
 }

 /**
  * Translates the world coordinate window of the target canvas in the 
  * vertical direction based on a given pan factor.
  * This factor is a percentage of the current window horizontal 
  * size. If the vertical Y axis is upward, a negative factor indicates
  * a upward window translation (the client model will move downward), 
  * and a positive factor indicates a downward window translation (the 
  * client model will move upward).  If the vertical Y axis is downward,
  * the reverse will occur.
  * <p>
  * The canvas is not automatically repainted by this method.
  * @param fac is the pan factor
  * @see #world_area
  * @see #setWorld
  */
 public void panWorldY( double fac )
 {
  double translation;            // window vertical translation
  double xmin, xmax, ymin, ymax; // new window limits

  translation = world_area.height * fac;
  xmin = world_area.x;
  xmax = world_area.x + world_area.width;
  ymin = world_area.y - translation;
  ymax = world_area.y + world_area.height - translation;
  setWorld( xmin, xmax, ymin, ymax );
 }

 /**
  * Scales the world coordinate window of the target canvas based on the 
  * given scaling factor. This factor is a percentage of decrease 
  * (positive factor) or increase (negative factor) of the window sizes.
  * The result of this inversion (positive ==> decrease, negative ==>
  * increase) is that the size of the client's enviroment 
  * decreases with a negative factor and increases with a positive
  * factor.
  * <p>
  * The canvas is not automatically repainted by this method.
  * @param fac is the scaling factor
  * @see #world_area
  * @see #setWorld
  */
 public void scaleWorld( double fac )
 {
  double scalefac = 1.0 + fac;   // scale factor from percentage factor
  double xc, yc;                 // center of world area
  double sizex, sizey;           // world area new half sizes

  xc = world_area.x + (world_area.width * 0.5);
  yc = world_area.y + (world_area.height * 0.5);
  sizex = (world_area.width*0.5) / scalefac;
  sizey = (world_area.height*0.5) / scalefac;
  setWorld( xc - sizex, xc + sizex, yc - sizey, yc + sizey );
 }

 /**
  * Sets the WC window vertical distortion factor.  This factor specifies
  * the amount of distortion between the height and width of the world
  * coordinate area. It is essentially a factor between the vertical and
  * horizontal scales in client world coordinate space. A factor with 
  * value greater than one will cause a vertical stretching of the 
  * image.
  * <p>
  * The canvas is not automatically repainted by this method.
  * @param vdistortion is the vertical distortion factor
  * @see #vdistortion
  * @see #setWorld
  */
 public void setVDistortion( double vdistortion )
 {
  if( vdistortion > 0.0 ) this.vdistortion = vdistortion;
 }

 /**
  * Gets the WC window vertical distortion factor.
  * @return vertical distortion factor
  * @see #vdistortion
  * @see #setVDistortion
  */
 public double getVDistortion()
 {
  return( vdistortion );
 }

 /**
  * Transforms the coordinates of a point given in client space (WC) to
  * device-dependent (raster) coordinates. The canvas device space has 
  * its origin in the upper-left corner of the canvas.
  * The <code>CanvasWC</code> object contains an {@link AffineTransform} 
  * object that defines how to convert coordinates from client space (world 
  * coordinates) to device-dependent (raster) coordinates in device space.
  * Note that, to avoid creating new point objects, the method returns a 
  * reference to a private object and not a copy of a new point.  If 
  * needed, clients must clone the returned object.
  * @param world_x is the x coordinate of given point in client space
  * @param world_y is the y coordinate of given point in client space
  * @return point in raster (device) coordinates
  * @see #setWorld
  * @see #raster2world
  */
 public Point world2raster( double world_x, double world_y )
 {
  AffineTransform wc2rc = g2off.getTransform();
  pt_world.x = world_x;
  pt_world.y = world_y;
  wc2rc.transform( pt_world, pt_world );
  pt_raster.x = (int)pt_world.x;
  pt_raster.y = (int)pt_world.y;
  return( pt_raster );
 }

 /**
  * Transforms the coordinates of a point given in device space (raster) to
  * client space (world coordinates). The canvas device space has 
  * its origin in the upper-left corner of the canvas.
  * The <code>CanvasWC</code> object contains an {@link AffineTransform} 
  * object that defines how to convert coordinates from device-dependent 
  * (raster) coordinates in device space to client space (world 
  * coordinates).
  * Note that, to avoid creating new point objects, the method returns a 
  * reference to a private object and not a copy of a new point.  If 
  * needed, clients must clone the returned object.
  * @param raster_x is the x coordinate of the given point in device space
  * @param raster_y is the y coordinate of the given point in device space
  * @return point in world (client) coordinates
  * @see #setWorld
  * @see #world2raster
  */
 public Point2D.Double raster2world( int raster_x, int raster_y )
 {
  AffineTransform wc2rc = g2off.getTransform();
  pt_world.x = (double)raster_x;
  pt_world.y = (double)raster_y;
  try {
   wc2rc.inverseTransform( pt_world, pt_world );
  }
  catch( NoninvertibleTransformException e ) {
   pt_world.x = 0.0;
   pt_world.y = 0.0;
  }
  return( pt_world );
 }

 /**
  * Handles the event of the user pressing down the mouse button.
  * Calls client method <code>mousePressedWC</code>
  * @param e is mouse event object
  * @see #mousePressedWC
  */
 public void mousePressed( MouseEvent e )
 {
  AffineTransform wc2rc = g2off.getTransform();
  ept_world.x = (double)e.getX();
  ept_world.y = (double)e.getY();
  try {
   wc2rc.inverseTransform( ept_world, ept_world );
  }
  catch( NoninvertibleTransformException error ) {
   ept_world.x = 0.0;
   ept_world.y = 0.0;
  }
  mousePressedWC( e, ept_world );
 }

 /**
  * Handles the event of the user dragging the mouse while holding down the mouse button.
  * Calls client method <code>mouseDraggedWC</code>
  * @param e is mouse event object
  * @see #mouseDraggedWC
  */
 public void mouseDragged( MouseEvent e )
 {
  AffineTransform wc2rc = g2off.getTransform();
  ept_world.x = (double)e.getX();
  ept_world.y = (double)e.getY();
  try {
   wc2rc.inverseTransform( ept_world, ept_world );
  }
  catch( NoninvertibleTransformException error ) {
   ept_world.x = 0.0;
   ept_world.y = 0.0;
  }
  mouseDraggedWC( e, ept_world );
 }

 /**
  * Handles the event of the user releasing the mouse button.
  * Calls client method <code>mouseReleasedWC</code>
  * @param e is mouse event object
  * @see #mouseReleasedWC  */
 public void mouseReleased( MouseEvent e )
 {
  AffineTransform wc2rc = g2off.getTransform();
  ept_world.x = (double)e.getX();
  ept_world.y = (double)e.getY();
  try {
   wc2rc.inverseTransform( ept_world, ept_world );
  }
  catch( NoninvertibleTransformException error ) {
   ept_world.x = 0.0;
   ept_world.y = 0.0;
  }
  mouseReleasedWC( e, ept_world );
 }

 // This method required by MouseListener.
 /**
  * Dummy method for the event of mouse motion in canvas.
  * @param e is mouse event object
  */
 public void mouseMoved( MouseEvent e ){}

 // This method is required by MouseMotionListener.
 /**
  * Dummy method for the event of mouse clicked in canvas.
  * @param e is mouse event object
  */
 public void mouseClicked( MouseEvent e ){}

 // This method is required by MouseMotionListener.
 /**
  * Dummy method for the event of mouse exited from canvas.
  * @param e is mouse event object
  */
 public void mouseExited( MouseEvent e ){}

 // This method is required by MouseMotionListener.
 /**
  * Dummy method for the event of mouse entered in canvas.
  * @param e is mouse event object
  */
 public void mouseEntered( MouseEvent e ){}

 /**
  * Repaint method of canvas. It calls method <code>update</code>.
  * @param g is the target graphics context
  * @see #update
  */
// public void paint( Graphics g )
// {
//  update( g );
// }

 /**
  * Update method of canvas.
  * <br>
  * In case the sizes of the canvas have changed, it (re)builds a 
  * {@link BufferedImage} object (offscreen image) to implement
  * double buffering rendering.  In this case, the world coordinate window 
  * defined by the client (if defined) is adjusted according to the new canvas
  * sizes.
  * <br>
  * Then, it clears the canvas using the currently defined background color and
  * calls the client's redraw method, passing the {@link Graphics2D} graphics 
  * context object of the offscreen image, so that the client may display
  * on it.
  * <br>
  * Finally, it fills the canvas with the image that was displayed by the canvas
  * on the offscreen image.	 
  * @param g is the target graphics context
  * @see #setWorld
  * @see #redraw
  */
 public void paintComponent( Graphics g )
 {
  if( (raster_area == null) || !raster_area.equals(getSize()) )
  {
   buildOffscreen();
   if( world_area == null )
   {
    world_area = new Rectangle2D.Double( 0.0, 0.0, 
                (double)raster_area.width, (double)raster_area.height );
    world_raw = (Rectangle2D.Double)world_area.clone();
    adjustWorld();
   }
   else
   {
    world_area.x = world_raw.x;
    world_area.y = world_raw.y;
    world_area.width = world_raw.width;
    world_area.height = world_raw.height;
    adjustWorld();
   }
  }

  // Clears the offscreen area.
  g2off.setColor( getBackground() );
  g2off.fill( world_area );

  // Calls abstract method for redrawing world.
  redraw( g2off );

  // Draws the buffered image to the screen.
  g.drawImage( ioff, 0, 0, this );
 }

 /**
  * Set font to be used to display texts.
  */
 public void setFont( Font fnt )
 {
  this.fnt = fnt;
  if( g2off != null )
   g2off.setFont( fnt );
 }

 /**
  * Set opacity to be used to display texts.
  */
 public void setTextOpacity( int text_opacity )
 {
  this.text_opacity = text_opacity;
 }

 /**
  * Set alignment to be used to display texts.
  */
 public void setTextAlignment( int text_alignment )
 {
  this.text_alignment = text_alignment;
 }

 /**
  * Draw a given string on a given position. 
  * (Parts of this method were stolen from TWFText.java)
  */
 public void drawString( String text, double x, double y )
 {
  TextLayout text_layout = new TextLayout( text, fnt, frc );
  Color curr_color = g2off.getColor();
  int w  = (int)text_layout.getAdvance();
  int as = (int)text_layout.getAscent();
  int ds = (int)text_layout.getDescent();
  int dx = 0;
  int dy = 0;

  switch( text_alignment )
  {
   case NORTH_WEST:
    dy -= as;
    break;
   case NORTH:
    dx -= w/2;
    dy -= as;
    break;
   case NORTH_EAST:
    dx -= w;
    dy -= as;
    break;
   case SOUTH_WEST:
    dy += ds;
    break;
   case SOUTH:
    dx -= w/2;
    dy += ds;
    break;
   case SOUTH_EAST:
    dx -= w;
    dy += ds;
   break;
  case WEST:
    dy -= (as - ds)/2;
    /* Deduction:
     *                _Top      _
     *    AAAA	                 |
     *   A    A                  | AS
     *   AAAAAA pppp             |
     *   A    A p   p            |
     *   A    A pppp  -BaseLine  -
     *          p                | DS
     *          p     _Bottom   _|
     *
     * AS is the ascent and DS is the descent of
     * the text.  To center the text in vertical  
     * direction we need to shift upward the 
     * baseline by AS, and downward by (AS+DS)/2.
     * So: dy = AS - (AS+DS)/2 = (AS-DS)/2.
     */
    break;
   case CENTER:
    dx -= w/ 2;
    dy -= (as - ds)/2;
    // look at "west" case
    break;
   case EAST:
    dx -= w;
    dy -= (as - ds)/2;
    // look at "west" case
    break;
   default:
   case BASE_LEFT:
    //dx = 0.0;
    //dy = 0.0; // trivial
    break;
   case BASE_CENTER:
    dx -= w/2;
    break;
   case BASE_RIGHT:
    dx -= w;
    break;
  }

  /* Finally draw the given text string. This is a little bit
   * tricky: The (Graphics2D) graphics context will transform
   * the text string with its current affine transformation 
   * matrix prior to displaying it.  To get the text with
   * its original font size, we temporarely set the current
   * transformation matrix to the identity matrix, draw the
   * text in raster coordinates, and restore the correct
   * affine transformation matrix.
   */
  pt_raster = world2raster( x, y );
  AffineTransform g2off_transf = g2off.getTransform();
  g2off.setTransform( identity );
  if( text_opacity == OPAQUE )
  {
   text_box.x = pt_raster.x + dx - 4;
   text_box.y = pt_raster.y - dy - as;
   text_box.width  = w + 7;
   text_box.height = as + ds;
   g2off.setColor( getBackground() );
   g2off.fill( text_box );
   g2off.setColor( curr_color );
  }
  g2off.drawString( text, pt_raster.x + dx, pt_raster.y - dy );
  g2off.setTransform( g2off_transf );
 }
}
